#!/usr/bin/perl -w
#
# registerSender.pl
#
# Copyright (C) 2011-2013 Cesnet z.s.p.o
#
# Use of this source is governed by a BSD-style license, see LICENSE file.

use strict;
use Getopt::Std;
use Switch;
use DBI;
use DBD::mysql;
use DateTime;
use Sys::Syslog qw(:DEFAULT setlogsock);
Sys::Syslog::setlogsock('unix');
use Carp;
use FindBin qw($RealBin $RealScript);
use lib "$FindBin::RealBin/../lib";
use WardenCommon;



################################################################################
#				VARIABLES
################################################################################
our $VERSION = "2.2";
my $etc = "$FindBin::RealBin/../etc";



################################################################################
#                               FUNCTIONS
################################################################################
sub usage {
  print "Usage: $RealScript [-h -n <hostname> -r <requestor> -s <service> -d <description_tags> -i <ip_net_client>]\n";
  exit 1;
}


sub help {
  print "$RealScript [-h -n <hostname> -r <requestor> -s <service> -d <description_tags> -i <ip_net_client>]\n";
  print "-h 	print this text and exit\n";
  print "-n 	hostname of sender\n";
  print "-r 	client registration requestor\n";
  print "-s 	name of service which sent events\n";
  print "-d 	description tags of send events\n";
  print "-i 	CIDR of sender\n";
  exit 0;
}



################################################################################
#                               MAIN
################################################################################
our ($opt_n, $opt_r, $opt_s, $opt_d, $opt_i, $opt_h);

if ($#ARGV == -1) {usage}

die usage unless getopts("n:r:s:d:i:h");
my $hostname		= $opt_n;
my $requestor		= $opt_r;
my $service		= $opt_s;
my $description_tags 	= $opt_d;
my $ip_net_client	= $opt_i;
my $help 		= $opt_h;

# catch help param
if ($help) {help}

# check validity of IP address (CIDR format)
if ($ip_net_client !~ /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/(\d|[1-2]\d|3[0-2]))$/) {
  WardenCommon::errMsg("Enter correct IP in CIDR format!");
}

# superuser controle
my $UID = $<;
if ($UID != 0) {WardenCommon::errMsg("You must be root for running this script!")}

# check parameters definition
switch () {
  case {!defined $hostname} {WardenCommon::errMsg("Parameter 'hostname' is not defined!")}
  case {!defined $requestor} {WardenCommon::errMsg("Parameter 'requestor' is not defined!")}
  case {!defined $service} {WardenCommon::errMsg("Parameter 'service' is not defined!")}
  case {!defined $description_tags} {WardenCommon::errMsg("Parameter 'description_tags' is not defined!")}
  case {!defined $ip_net_client} {WardenCommon::errMsg("Parameter 'ip_net_client' is not defined!")}
}

# defined variables by method
my $client_type		= "s";
my $valid		= "t";
my $registered		= DateTime->now;
my $type		= undef;
my $receive_own_events	= undef;

# read config file
my $conf_file = "$etc/warden-server.conf";
our $SYSLOG             = undef;
our $SYSLOG_VERBOSE     = undef;
our $SYSLOG_FACILITY    = undef;
our $DB_NAME            = undef;
our $DB_USER            = undef;
our $DB_PASS            = undef;
our $DB_HOST            = undef;
unless (do $conf_file) {
  die("Errors in config file '$conf_file': $@") if $@;
  die("Can't read config file '$conf_file': $!") unless defined $_;
  # if $_ defined, it's retvalue of last statement of conf, for which we don't care
}

# connect to DB
our $DBH = DBI->connect("DBI:mysql:database=$DB_NAME;host=$DB_HOST", $DB_USER, $DB_PASS, {RaiseError => 1, mysql_auto_reconnect => 1})
           || die "Could not connect to database: $DBH->errstr";

# check if receiver has been already registered or register it
my $sth = $DBH->prepare("SELECT registered FROM clients WHERE hostname = ? AND service = ? AND client_type = ? AND ip_net_client = ? LIMIT 1;") or die "Cannot prepare statement: " . $DBH->errstr;
$sth->execute($hostname, $service, $client_type, $ip_net_client) or die "Cannot execute statement: " . $sth->errstr;
my $result = $sth->fetchrow();
my $warden_server = WardenCommon::trim(`hostname -f`);
if (defined $result) {
  WardenCommon::sendMsg($SYSLOG, $SYSLOG_VERBOSE, $SYSLOG_FACILITY, "err", "Attempt to re-register the sender [hostname: '$hostname', service: '$service', cidr: '$ip_net_client']");
  WardenCommon::errMsg("Sender has already been registered at $warden_server in '$result'");
} else {
  $sth = $DBH->prepare("INSERT INTO clients VALUES (?,?,?,?,?,?,?,?,?,?,?);") or die "Cannot prepare statement: " . $DBH->errstr;
  my $ret_val = $sth->execute(undef, $hostname, $registered, $requestor, $service, $client_type, $type, $receive_own_events, $description_tags, $ip_net_client, $valid) or die "Cannot execute statement: " . $sth->errstr;
  if ($ret_val == 1) {
    WardenCommon::sendMsg($SYSLOG, $SYSLOG_VERBOSE, $SYSLOG_FACILITY, "info", "New sender [hostname: '$hostname', service: '$service', cidr: '$ip_net_client'] was registered");
    WardenCommon::succMsg("Registration of sender from $hostname was SUCCESSFUL!!!");
  } else {
    WardenCommon::errMsg("Registration of sender from $hostname FAILED!!!");
  }
}
